package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.collections.objectpool.IPoolObj;
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import flash.display.Shape;
	import flash.geom.Point;
	
	/**
	 * Points the direction to a planet
	 * @author Damian Connolly
	 */
	public class DirArrow extends Shape implements IUpdateObj, IPoolObj
	{
		public var dir:Point 		= new Point;
		public var ship:Ship		= null;
		public var planet:Planet	= null;
		
		public function get active():Boolean { return this.visible && ( this.ship != null && !this.ship.isPaused ); }
		
		public function DirArrow() 
		{
			this.draw();
			DS.addToUpdate( this );
			this.visible = false;
		}
		
		public function init(params:* = null):void 
		{
			this.planet = params as Planet;
			this.planet.parent.addChild( this );
			
			// show it so it updates
			this.visible = true;
		}
		
		public function destroy():void 
		{
			DS.removeFromUpdate( this );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.ship	= null;
			this.planet	= null;
			this.dir	= null;
		}
		
		public function reset():void 
		{
			this.planet		= null;
			this.visible 	= false;
			if ( this.parent != null )
				this.parent.removeChild( this );
		}
		
		public function update( dt:Number ):void
		{
			if ( this.ship == null )
				return;
			
			// rotate towards the planet
			this.dir.x 	= ( this.planet.x - this.ship.x );
			this.dir.y 	= ( this.planet.y - this.ship.y );
			this.dir.normalize( 1.0 );
			
			var rads:Number	= MathHelper.radianAngle( this.dir );
			this.rotation 	= MathHelper.radiansToDegrees( rads );
			
			// set the alpha - further away, lighter
			var alphaDist:Number = MathHelper.clamp( this.planet.zPos, SpaceItem.MAX_DIST * 0.1, SpaceItem.MAX_DIST );
			this.alpha = 0.2 + 0.8 * ( 1.0 - ( alphaDist / SpaceItem.MAX_DIST ) );
			
			// position
			var desiredX:Number = this.ship.x + Math.cos( rads ) * 65;
			var desiredY:Number = this.ship.y + Math.sin( rads ) * 65;
			this.x += ( desiredX - this.x ) * 0.5;
			this.y += ( desiredY - this.y ) * 0.5;
		}
		
		public function draw():void
		{
			var aw:Number = 10.0;
			var ah:Number = 20.0;
			this.graphics.clear();
			this.graphics.lineStyle( 1.0, 0xffffff );
			this.graphics.drawCircle( -aw * 0.25, 0.0, ah );
			this.graphics.moveTo( -aw * 0.5, -ah * 0.5 );
			this.graphics.lineTo( aw * 0.5, 0.0 );
			this.graphics.lineTo( -aw * 0.5, ah * 0.5 );
		}
		
	}

}