package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.collections.objectpool.IPoolObj;
	import com.divillysausages.ds.collections.objectpool.ObjectPool;
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.util.Colour;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Graphics;
	import flash.display.Shape;
	import flash.display.Sprite;
	
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Particle extends Sprite implements IUpdateObj, IPoolObj
	{
		
		/*******************************************************************************************/
		
		private static var m_particleBMD:BitmapData 	= null;	// the image for our particle
		private static var m_pool:ObjectPool 			= null; // the pool for this particle
		private static var m_active:Vector.<Particle>	= new Vector.<Particle>;
		
		/*******************************************************************************************/
		
		public static function showParticle( ship:Ship, life:Number ):void
		{
			if ( Particle.m_pool == null )
				Particle.m_pool = new ObjectPool( Particle, 0, false );
				
			var p:Particle 	= Particle.m_pool.checkOut() as Particle;
			p.x				= ship.x;
			p.y				= ship.y;
			p.rotation		= ship.totalRotation;
			p.m_time		= ( life <= 0.0 ) ? 0.1 : life;
			p.m_yVel		= -ship.dir.y * 2.0;
			
			var depth:int = ship.parent.getChildIndex( ship ) + 1;
			ship.parent.addChildAt( p, depth );
			
			Particle.m_active.push( p );
		}
		
		public static function pauseAllActiveParticle():void
		{
			for each( var p:Particle in Particle.m_active )
				p.isPaused = true;
		}
		
		public static function clearParticles():void
		{
			for each( var p:Particle in Particle.m_active )
				Particle.m_pool.checkIn( p );
			Particle.m_active.length = 0;
		}
		
		/*******************************************************************************************/
		
		public var isPaused:Boolean = false;
		
		/*******************************************************************************************/
		
		private var m_currTime:Number	= 0.0;
		private var m_time:Number		= 0.0;
		private var m_yVel:Number		= 0.0;
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return ( this.parent != null ) && !this.isPaused; }
		
		/*******************************************************************************************/
		
		public function Particle() 
		{		
			if ( Particle.m_particleBMD == null )
			{
				// draw our particle
				var width:Number	= 40.0;
				var height:Number	= 30.0;
				var s:Shape 		= new Shape;
				this._drawParticlePart( s.graphics, Colour.WHITE.rgb, 0.2, width, height, width, height );
				this._drawParticlePart( s.graphics, Colour.WHITE.rgb, 0.4, width * 0.6, height * 0.6, width, height );
				this._drawParticlePart( s.graphics, Colour.fromBlend( Colour.WHITE, Colour.LIGHT_BLUE, 0.2 ).rgb, 1.0, width * 0.3, height * 0.3, width, height );
				
				// convert to a bmd
				Particle.m_particleBMD = new BitmapData( width, height, true, 0x00000000 );
				Particle.m_particleBMD.draw( s );
			}
				
			// graphics
			var b:Bitmap 	= new Bitmap( Particle.m_particleBMD );
			b.x				= -b.width * 0.5;
			b.y				= -b.height * 0.5;
			this.addChild( b );
			
			// reset
			this.reset();
			
			// add to the update
			DS.addToUpdate( this );
		}
		
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			if ( this.parent != null )
				this.parent.removeChild( this );
		}
		
		public function init(params:* = null):void 
		{
			// not used
		}
		
		public function reset():void
		{
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.isPaused	= false;
			this.alpha 		= 1.0;
			this.scaleX		= 1.0;
			this.scaleY		= 1.0;
			this.m_currTime	= 0.0;
		}
		
		public function update( dt:Number ):void
		{
			// move down
			this.y 		+= this.m_yVel;
			this.m_yVel += dt;
			
			this.m_currTime += dt;
			if ( this.m_currTime > this.m_time )
				this.m_currTime = this.m_time;
				
			// set our alpha
			var pc:Number	= this.m_currTime / this.m_time;
			this.alpha 		= 1.0 - pc;
			
			// if we're dead, then re-add to the pool
			if ( this.m_currTime >= this.m_time )
			{
				Particle.m_pool.checkIn( this );
				var index:int = Particle.m_active.indexOf( this );
				if ( index != -1 )
					Particle.m_active.splice( index, 1 );
			}
		}
		
		/*******************************************************************************************/
		
		private function _drawParticlePart( graphics:Graphics, colour:uint, alpha:Number, w:Number, h:Number, tw:Number, th:Number ):void
		{
			graphics.beginFill( colour, alpha );
			graphics.moveTo( tw * 0.5 - w * 0.5, th * 0.5 ); // left corner
			graphics.lineTo( tw * 0.5, th * 0.5 - h * 0.5 ); // top
			graphics.lineTo( tw * 0.5 + w * 0.5, th * 0.5 ); // right
			graphics.lineTo( tw * 0.5, th * 0.5 + h * 0.5 ); // bottom
			graphics.lineTo( tw * 0.5 - w * 0.5, th * 0.5 ); // left corner
			graphics.endFill();
		}
		
	}

}