package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.collections.objectpool.ObjectPool;
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.math.MathHelper;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.geom.Point;
	
	/**
	 * The ship that we're going to use
	 * @author Damian Connolly
	 */
	public class Ship extends Sprite implements IUpdateObj, ISortable
	{
		
		/*******************************************************************************************/
		
		/**
		 * The dir that we're heading in, from -1.0 to 1.0, based on the center of the screen
		 */
		public var dir:Point = null;
		
		/**
		 * Are we paused?
		 */
		public var isPaused:Boolean = false;
		
		/*******************************************************************************************/
		
		private var m_graphics:Shape			= null;
		private var m_origPoints:Vector.<Point> = new Vector.<Point>;
		private var m_tranPoints:Vector.<Point> = new Vector.<Point>;
		private var m_arrows:Vector.<DirArrow>	= new Vector.<DirArrow>; // currently active arrows
		private var m_arrowPool:ObjectPool		= null;		// for showing where the planets are
		private var m_isLooping:Boolean 		= false;	// are we currently looping?
		private var m_loopingAngle:Number 		= 0.0;		// our current looping angle
		private var m_loopingDir:int 			= 0;		// the direction that we're looping in
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return ( this.parent != null ); }
		
		public function get zPos():Number { return -1.0; }
		
		public function get totalRotation():Number { return this.rotation + this.m_graphics.rotation; }
		
		/*******************************************************************************************/
		
		public function Ship() 
		{			
			this.dir 		= new Point;
			this.m_graphics	= new Shape;
			this.addChild( this.m_graphics );
			
			// create our graphics points
			var len:int			= 5;
			this.m_origPoints 	= new Vector.<Point>( len, true );
			this.m_tranPoints 	= new Vector.<Point>( len, true );
			for ( var i:int = 0; i < len; i++ )
			{
				this.m_origPoints[i] = new Point;
				this.m_tranPoints[i] = new Point;
			}
			
			// create our dir arrows
			this.m_arrowPool = new ObjectPool( DirArrow, 3, false );
			
			// add to update
			DS.addToUpdate( this );
		}
		
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			
			this.removeChildren( 0, int.MAX_VALUE );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			for each( var arrow:DirArrow in this.m_arrows )
				this.m_arrowPool.checkIn( arrow );
			this.m_arrowPool.destroy();
				
			this.dir			= null;
			this.m_origPoints	= null;
			this.m_tranPoints	= null;
			this.m_arrows		= null;
			this.m_arrowPool	= null;
			this.m_graphics		= null;
		}
		
		// called when a new planet has appeared
		public function onNewPlanet( planet:Planet ):void
		{
			var arrow:DirArrow 	= ( this.m_arrowPool.checkOut() as DirArrow );
			arrow.ship			= this;
			arrow.init( planet );
			this.m_arrows.push( arrow );
		}
		
		// called when a planet has been removed
		public function onRemovePlanet( planet:Planet ):void
		{
			// find our arrow
			var len:int = this.m_arrows.length;
			for ( var i:int = 0; i < len; i++ )
			{
				if ( this.m_arrows[i].planet == planet )
				{
					this.m_arrowPool.checkIn( this.m_arrows[i] );
					this.m_arrows.splice( i, 1 );
					return;
				}
			}
		}
		
		public function update( dt:Number ):void
		{
			if ( this.isPaused )
				return;
				
			// update our dir
			this.dir.x = ( Input.mousePos.x - DS.stage.stageWidth * 0.5 ) / ( DS.stage.stageWidth * 0.5 );
			this.dir.y = ( Input.mousePos.y - DS.stage.stageHeight * 0.5 ) / ( DS.stage.stageHeight * 0.5 );
			
			// lerp out a bit to make our ship more dynamic
			this.x = MathHelper.lerp( this.x, DS.stage.stageWidth * 0.5 + DS.stage.stageWidth * 0.5 * this.dir.x, dt );
			this.y = MathHelper.lerp( this.y, DS.stage.stageHeight - 100.0 + 40.0 * this.dir.y, dt );
			
			// draw our ship
			this._draw();
				
			// mouse to loop
			if ( Input.isMousePressed )
			{
				this.m_isLooping = true;
				if ( this.m_loopingAngle == 0.0 )
					this.m_loopingDir = ( this.dir.x < 0.0 ) ? -1 : 1;
			}
			if ( this.m_isLooping )
			{
				this.m_loopingAngle += 360.0 * dt * 3.0 * this.m_loopingDir;
				this.m_graphics.rotation = ( this.m_loopingAngle < -360.0 ) ? -360.0 : ( this.m_loopingAngle > 360.0 ) ? 360.0 : this.m_loopingAngle;
				if ( this.m_loopingAngle <= -360.0 || this.m_loopingAngle >= 360.0 )
				{
					this.m_isLooping 	= false;
					this.m_loopingAngle = 0.0;
				}
			}
		}
		
		/*******************************************************************************************/
		
		// draw our ship
		private function _draw():void
		{
			this.m_graphics.graphics.clear();
			this.m_graphics.graphics.lineStyle( 1.0, 0xffffff );
			
			// normalise our dir y for our graphics
			var dirY:Number = ( this.dir.y - 1.0 ) * 0.5;
			
			// the size of the ship
			var shipBackW:Number 	= 80.0;
			var shipBackH:Number 	= 40.0;
			var noseH:Number		= 40.0;
			var noseHorDev:Number	= 40.0;	// max hor deviation for the nose
			var maxRot:Number		= 80.0; // max rotation based on x
			var hw:Number			= shipBackW * 0.5;
			var hh:Number			= shipBackH * 0.5;
			
			// draw the ship depending on where the mouse it in relation to the center of the screen
			var multX:Number	= ( this.dir.x < 0.0 ) ? -1.0 : 1.0;
			var multY:Number	= ( dirY < 0.0 ) ? -1.0 : 1.0;
			var shipMidY:Number	= hh * 0.75 + ( hh * 0.25 * ( 1.0 - Math.abs( dirY ) ) ); // the further we are, then smaller the ship height
			
			// set our original and translated points
			var bl1:Point 	= this.m_origPoints[0]; // back left
			var bt1:Point 	= this.m_origPoints[1]; // back top
			var br1:Point 	= this.m_origPoints[2]; // back right
			var bb1:Point 	= this.m_origPoints[3]; // back bottom
			var n1:Point	= this.m_origPoints[4];	// nose
			
			var bl2:Point 	= this.m_origPoints[0]; // back left
			var bt2:Point 	= this.m_origPoints[1]; // back top
			var br2:Point 	= this.m_origPoints[2]; // back right
			var bb2:Point 	= this.m_origPoints[3]; // back bottom
			var n2:Point	= this.m_origPoints[4];	// nose
			
			var noseX:Number 	= 0.0;// noseHorDev * this.dir.x;
			var noseY:Number 	= shipMidY * multY + noseH * dirY * ( 0.8 + Math.abs( this.dir.x * 0.2 ) );
			var radians:Number	= 0.0;// MathHelper.degreesToRadians( maxRot * this.dir.x * -dirY );
			bl1.setTo( -hw, 0.0 );			bl2 = MathHelper.rotate( bl1, radians, bl2 );
			bt1.setTo( 0.0, -shipMidY );	bt2 = MathHelper.rotate( bt1, radians, bt2 );
			br1.setTo( hw, 0.0 );			br2 = MathHelper.rotate( br1, radians, br2 );
			bb1.setTo( 0.0, shipMidY );		bb2 = MathHelper.rotate( bb1, radians, bb2 );
			n1.setTo( noseX, noseY );		n2 	= MathHelper.rotate( n1, radians, n2 );
			
			// nose outline
			this.m_graphics.graphics.beginFill( DS.stage.color );
			this.m_graphics.graphics.moveTo( bl2.x, bl2.y );
			this.m_graphics.graphics.lineTo( n2.x, n2.y );
			this.m_graphics.graphics.lineTo( br2.x, br2.y );
			this.m_graphics.graphics.lineTo( bt2.x, bt2.y );
			this.m_graphics.graphics.lineTo( bl2.x, bl2.y );
			this.m_graphics.graphics.endFill();
			
			// nose center line
			this.m_graphics.graphics.moveTo( n2.x, n2.y );
			this.m_graphics.graphics.lineTo( ( dirY < 0.0 ) ? bt2.x : bb2.x, ( dirY < 0.0 ) ? bt2.y : bb2.y );
			
			// back
			this.m_graphics.graphics.beginFill( DS.stage.color );
			this.m_graphics.graphics.moveTo( bl2.x, bl2.y );
			this.m_graphics.graphics.lineTo( bt2.x, bt2.y );
			this.m_graphics.graphics.lineTo( br2.x, br2.y );
			this.m_graphics.graphics.lineTo( bb2.x, bb2.y );
			this.m_graphics.graphics.lineTo( bl2.x, bl2.y );
			this.m_graphics.graphics.endFill();
			
			// rotate our ship a bit
			this.rotation = maxRot * this.dir.x;
		}
		
	}

}