package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import flash.display.Sprite;
	import flash.geom.Point;
	
	/**
	 * Base class for items in space
	 * @author Damian Connolly
	 */
	public class SpaceItem extends Sprite implements IUpdateObj, ISortable
	{
		
		/*******************************************************************************************/
		
		public static const MAX_DIST:Number		= 1000.0;
		public static const MIN_SCALE:Number	= Math.log( 0.01 );
		public static const MAX_SCALE:Number	= Math.log( 7.0 );
		
		/*******************************************************************************************/
		
		private static var M_SHIP_RIGHT:Point 	= new Point;
		private static var M_PROJ:Point			= new Point;
		private static var M_HELPER_POINT:Point	= new Point;
		
		/*******************************************************************************************/
		
		public var isPaused:Boolean = false;
		
		/*******************************************************************************************/
		
		protected var m_ship:Ship 					= null; // the ship that we're tracking
		protected var m_shipDir:Point				= null;	// the direction the ship is going in
		protected var m_zPos:Number					= 0.0;	// the z position of the SpaceItem
		protected var m_velZ:Number					= 0.0;	// how fast to move on the z axis
		protected var m_shouldCheckBounds:Boolean	= false;// should we check bounds in case we've moved too far?
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return ( this.parent != null && !this.isPaused ); }
		
		public function get scale():Number { return this.scaleX; }
		public function set scale( n:Number ):void
		{
			// we have a minimum scale (otherwise our bmd for the filter is fucked)
			if ( n < 0.005 && n > -0.005 )
				n = 0.005;
				
			// if it's too similar, do nothing
			if ( Math.abs( n - this.scaleX ) < 0.01 )
				return;
				
			this.scaleX = n;
			this.scaleY = n;
		}
		
		public function get zPos():Number { return this.m_zPos; }
		public function set zPos( n:Number ):void
		{
			this.m_zPos 		= n;
			var pc:Number		= 1.0 - ( this.m_zPos / 1000 );
			var scale:Number	= Math.exp( SpaceItem.MIN_SCALE + ( SpaceItem.MAX_SCALE - SpaceItem.MIN_SCALE ) * pc );
			if ( this.m_zPos <= 0.0 )
			{
				// call our method and reset
				if( this._onFinish() )
					this.reset();
			}
			else
				this.scale = scale;
		}
		
		/*******************************************************************************************/
		
		public function SpaceItem( ship:Ship ) 
		{
			// set our properties
			this.mouseEnabled		= false;
			this.mouseChildren		= false;
			this.m_shipDir 			= new Point;
			this.m_ship 			= ship;
			
			// set our initial z pos (creates the filter)
			this.zPos = SpaceItem.MAX_DIST;
			
			// add us to the update
			DS.addToUpdate( this );
		}
		
		public function destroy():void 
		{
			DS.removeFromUpdate( this );
			
			this.removeChildren( 0, int.MAX_VALUE );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.m_ship		= null;
			this.m_shipDir	= null;
		}
		
		public function reset():void
		{
			this.isPaused	= false;
			this.zPos		= SpaceItem.MAX_DIST; // also sets scale
			this.x			= Math.random() * DS.stage.stageWidth;
			this.y			= Math.random() * DS.stage.stageHeight;
			this.m_velZ		= 0.0;
		}
		
		public function update( dt:Number ):void
		{				
			// get our dir and normalise it
			this.m_shipDir.setTo( this.m_ship.dir.x, this.m_ship.dir.y );
			this.m_shipDir.normalize( 1.0 );
			
			this.zPos 	-= this.m_velZ;
			this.m_velZ += dt;
			if ( this.m_velZ > 5.0 )
				this.m_velZ = 5.0;
			if ( this.m_ship == null ) // if the ship is null, then we've been destroyed
				return;
			
			// move the SpaceItem
			var pc:Number = ( 1.0 - ( this.m_zPos / SpaceItem.MAX_DIST ) );
			var mv:Number = pc * ( 5.0 + ( 50.0 * pc ) );
			if ( mv < 2.0 ) mv = 2.0;	
			this.x += -this.m_shipDir.x * mv * Math.abs( this.m_ship.dir.x );
			this.y += -this.m_shipDir.y * mv * Math.abs( this.m_ship.dir.y );
			
			// if we've moved too far, correct to (to mimic flying in a circle) (take the right vector of the ship dir, 
			// and project us onto it, then flip us if the dist is too great
			if ( this.m_shouldCheckBounds )
			{
				SpaceItem.M_HELPER_POINT.setTo( this.x - DS.stage.stageWidth * 0.5, this.y - DS.stage.stageHeight * 0.5 );
				SpaceItem.M_SHIP_RIGHT 		= MathHelper.right( this.m_ship.dir, SpaceItem.M_SHIP_RIGHT );
				SpaceItem.M_PROJ 			= MathHelper.proj( SpaceItem.M_HELPER_POINT, SpaceItem.M_SHIP_RIGHT, SpaceItem.M_PROJ );
				SpaceItem.M_HELPER_POINT.x 	= DS.stage.stageWidth * 0.5 + SpaceItem.M_PROJ.x - this.x;
				SpaceItem.M_HELPER_POINT.y 	= DS.stage.stageHeight * 0.5 + SpaceItem.M_PROJ.y - this.y;
				var max:Number 				= ( DS.stage.stageWidth > DS.stage.stageHeight ) ? DS.stage.stageWidth * 2.0 : DS.stage.stageHeight * 2.0;
				if ( SpaceItem.M_HELPER_POINT.length > max )
				{
					SpaceItem.M_HELPER_POINT.normalize( max * 2.0 );
					this.x += SpaceItem.M_HELPER_POINT.x;
					this.y += SpaceItem.M_HELPER_POINT.y;
				}
			}
		}
		
		/*******************************************************************************************/
		
		// called when we've finished moving - remove us, or do something special
		protected function _onFinish():Boolean
		{
			return true;
		}
		
	}

}