package com.divillysausages.connected.logic 
{
	import com.divillysausages.connected.assets.Resources;
	import com.divillysausages.connected.constants.SoundConstants;
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.greensock.easing.Back;
	import com.greensock.TweenLite;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * the speech for the player
	 * @author Damian Connolly
	 */
	public class Speech extends Sprite implements IUpdateObj
	{
		
		/*******************************************************************************************/
		
		public var isFullyShown:Boolean = false;
		
		/*******************************************************************************************/
		
		private var m_txt:TextField				= null; // the textfield for us
		private var m_msg:String				= null;	// the message that we're showing
		private var m_maxW:Number				= 0.0;	// the max width for our message
		private var m_count:int					= 0;	// how many letters we're currently showing
		private var m_arrowW:Number				= 5.0;	// the width of our arrow
		private var m_paddingH:Number			= 10.0;	// padding between our text and bg (horizontal)
		private var m_paddingV:Number			= 5.0;	// padding between our text and bg (vertical)
		private var m_fadingIn:Boolean			= false;// are we fading in?
		private var m_fadingOut:Boolean			= false;// are we fading out?
		private var m_showImmediately:Boolean	= false;// should we show the full message right away?
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return this.parent != null; }
		
		/*******************************************************************************************/
		
		public function Speech( maxW:Number ) 
		{
			this.m_maxW = maxW - this.m_paddingH * 2.0;
			this.alpha	= 0.0; // hide at the start
			
			var tf:TextFormat 				= new TextFormat( Resources.GAME_FONT, 25, 0xffffff );
			this.m_txt						= new TextField;
			this.m_txt.autoSize				= TextFieldAutoSize.LEFT;
			this.m_txt.defaultTextFormat	= tf;
			this.m_txt.multiline			= false;
			this.m_txt.wordWrap				= false;
			this.m_txt.mouseEnabled			= false;
			this.m_txt.mouseWheelEnabled	= false;
			this.m_txt.embedFonts			= true;
			this.m_txt.x					= this.m_arrowW + this.m_paddingH;
			this.m_txt.y					= this.m_paddingV * 0.7; // vertical space seems a bit fucked at the top
			this.addChild( this.m_txt );
			
			// add to update
			DS.addToUpdate( this );
		}
		
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			this.removeChildren( 0, int.MAX_VALUE );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.m_txt	= null;
		}
		
		public function clear():void
		{
			this.m_txt.text			= "";
			this.m_msg				= null;
			this.m_count			= 0;
			this.m_fadingIn			= false;
			this.m_fadingOut		= false;
			this.alpha				= 0.0;
			this.m_showImmediately	= false;
		}
		
		public function show( msg:String ):void
		{
			// set our initial message
			this.m_msg 			= msg;
			this.isFullyShown	= false;
			if ( this.alpha > 0.0 ) // we're currently showing a message, so hide that one first
			{
				this.m_fadingIn		= false;
				this.m_fadingOut 	= true;
				return;
			}
			
			// reset the text
			this.m_txt.autoSize		= TextFieldAutoSize.LEFT;
			this.m_txt.multiline 	= false;
			this.m_txt.wordWrap		= false;
			this.m_txt.text			= msg;
			this.m_count			= 0;
			
			// if our text is too long, break it up
			if ( this.m_txt.width > this.m_maxW )
			{
				this.m_txt.autoSize		= TextFieldAutoSize.NONE;
				this.m_txt.multiline 	= true;
				this.m_txt.wordWrap		= true;
				this.m_txt.width		= this.m_maxW;
				this.m_txt.height		= this.m_txt.textHeight + 4.0;
			}
			
			// draw our bg
			var w:Number 	= this.m_txt.width + this.m_paddingH * 2.0;
			var h:Number 	= this.m_txt.height + this.m_paddingV * 2.0;
			var aw:Number	= this.m_arrowW; 	// arrow width
			var ah:Number	= 10.0;				// arrow height
			var ap:Number	= 15.0;				// arrow padding
			this.graphics.clear();
			this.graphics.lineStyle( 1.0, 0xffffff );
			this.graphics.beginFill( 0xffffff, 0.5 );
			this.graphics.moveTo( aw, 0.0 );
			this.graphics.lineTo( aw + w, 0.0 );
			this.graphics.lineTo( aw + w, h );
			this.graphics.lineTo( aw, h );
			this.graphics.lineTo( aw, ah + ap );
			this.graphics.lineTo( 0.0, ah * 0.5 + ap );
			this.graphics.lineTo( aw, ap );
			this.graphics.lineTo( aw, 0.0 );
			this.graphics.endFill();
			
			// clear the text (we'll fill it in the update)
			this.m_txt.text 	= "";
			this.m_fadingIn		= true;
			this.m_fadingOut	= false;
			
			// if we're showing immediately, do it
			if ( this.m_showImmediately )
			{
				this.showImmediately();
				this.m_showImmediately = false;
			}
		}
		
		public function showImmediately():void
		{
			if ( this.m_msg == null )
				return;
				
			// if we're fading out, show the next one
			if ( this.m_fadingOut )
			{
				this.m_showImmediately = true;
				return;
			}
			
			// show the text
			this.m_txt.text = this.m_msg;
			this.m_msg		= null;
			if ( this.alpha >= 1.0 && !this.m_fadingIn )
				this.isFullyShown = true;
		}
		
		public function hide():void
		{
			this.m_fadingIn 	= false;
			this.m_fadingOut	= true;
			this.m_msg			= null; // otherwise we'll just show it again
		}
		
		public function update( dt:Number ):void
		{
			// if we're fading in
			if ( this.m_fadingIn )
			{
				this.alpha += dt * 6.0;
				if ( this.alpha >= 1.0 )
				{
					this.alpha 			= 1.0;
					this.m_fadingIn 	= false;
					this.isFullyShown	= true;
				}
				else
					return;
			}
			
			// if we're fading out
			if ( this.m_fadingOut )
			{
				this.alpha -= dt * 6.0;
				if ( this.alpha <= 0.0 )
				{
					this.alpha 			= 0.0;
					this.m_fadingOut	= false;
					if ( this.m_msg != null )
						this.show( this.m_msg );
				}
				return;
			}
			
			if ( this.m_msg == null )
				return;
				
			this.m_count += 2;
			this.m_txt.text = this.m_msg.substr( 0, this.m_count );
			if ( this.m_count >= this.m_msg.length )
				this.m_msg = null;
				
			DS.soundManager.playSound( SoundConstants.SFX_TEXT, 0, 0.2 );
		}
		
	}

}