package com.divillysausages.connected.state
{
	import com.divillysausages.connected.assets.Resources;
	import com.divillysausages.connected.state.constants.StateNames;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.security.SafeNumber;
	import flash.display.Bitmap;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * The state where we show up our logo
	 * @author Damian Connolly
	 */
	public class StateDS extends State 
	{
		
		/*******************************************************************************************/
		
		private var m_logo:Bitmap 			= null;	// our logo
		private var m_text:TextField		= null;	// our text
		private var m_isFadingIn:Boolean	= false;// are we fading in?
		private var m_isFadingOut:Boolean	= false;// are we fading out?
		private var m_isWaiting:Boolean		= false;
		private var m_time:SafeNumber		= null;	// the time to hold after the fade in
		private var m_currTime:SafeNumber	= null;	// our current time
		
		/*******************************************************************************************/
		
		/**
		 * Creates our DS state
		 * @param fsm The Game finite state machine
		 */
		public function StateDS( fsm:Game )
		{
			super( fsm, StateNames.STATE_DS );
			
			// our time objects
			this.m_time 	= new SafeNumber( 1.0 );
			this.m_currTime	= new SafeNumber( 0.0 );
		}
		
		/**
		 * Starts the DS state
		 */
		override public function begin():void 
		{
			var main:Main = ( DS.main as Main );
			
			// create our logo
			this.m_logo 			= new Bitmap;
			this.m_logo.bitmapData	= DS.getAsset( Resources.DS_IMAGE ).bitmapData;
			this.m_logo.x			= DS.stage.stageWidth * 0.5 - this.m_logo.width * 0.5;
			this.m_logo.y			= DS.stage.stageHeight * 0.5 - this.m_logo.height * 0.5 - 20;
			this.m_logo.alpha		= 0.0;
			main.addChild( this.m_logo );
			
			// create our text
			var tf:TextFormat 				= new TextFormat( Resources.GAME_FONT, 15, 0xffffff );
			this.m_text 					= new TextField;
			this.m_text.defaultTextFormat 	= tf;
			this.m_text.autoSize			= TextFieldAutoSize.LEFT;
			this.m_text.htmlText			= "<a href='http://divillysausages.com' target='_blank'>http://divillysausages.com</a>";
			this.m_text.selectable			= false;
			this.m_text.mouseEnabled		= true;
			this.m_text.mouseWheelEnabled	= false;
			this.m_text.embedFonts			= true;
			this.m_text.x					= DS.stage.stageWidth * 0.5 - this.m_text.width * 0.5;
			this.m_text.y					= this.m_logo.y + this.m_logo.height - 40.0;
			this.m_text.alpha				= 0.0;
			main.addChild( this.m_text );
			
			// reset our vars
			this.m_currTime.value 	= 0.0;
			this.m_isFadingIn		= true;
		}
		
		/**
		 * Called when we're leaving this state
		 */
		override public function end():void 
		{
			this.m_logo.parent.removeChild( this.m_logo );
			this.m_text.parent.removeChild( this.m_text );
			this.m_logo = null;
			this.m_text = null;
		}
		
		/**
		 * Shows our logo for a period of time
		 * @param dt The time since the last call to update
		 */
		override public function update(dt:Number):void 
		{
			// if we're not fading in or out, do nothing
			if ( !this.m_isFadingIn && !this.m_isFadingOut && !this.m_isWaiting )
				return;
				
			// if we don't have anything to fade, do nothing
			if ( this.m_logo == null || this.m_text == null )
				return;
				
			// fade in
			if ( this.m_isFadingIn )
			{
				this.m_logo.alpha += 0.05;
				this.m_text.alpha += 0.05;
				if ( this.m_logo.alpha >= 1.0 )
				{
					this.m_logo.alpha 	= 1.0;
					this.m_text.alpha 	= 1.0;
					
					if( this.m_isFadingIn )
						DS.soundManager.playSound( Resources.DS_SOUND );
					
					this.m_isFadingIn	= false;
					this.m_isFadingOut 	= false;
					this.m_isWaiting	= true;
				}
			}
			
			if ( this.m_isWaiting )
			{
				this.m_currTime.value += dt;
				if ( this.m_currTime.value > this.m_time.value )
				{
					this.m_isWaiting 	= false;
					this.m_isFadingIn	= false;
					this.m_isFadingOut	= true;
				}
			}
			
			// fade out
			if ( this.m_isFadingOut )
			{
				this.m_logo.alpha -= 0.05;
				this.m_text.alpha -= 0.05;
				if ( this.m_logo.alpha <= 0.0 )
				{
					this.m_logo.alpha 	= 0.0;
					this.m_text.alpha 	= 0.0;
					this.m_isFadingIn	= false;
					this.m_isFadingOut	= false;
					
					// move on
					this.m_fsm.changeState( StateNames.STATE_PLAY );
				}
			}
		}
		
	}

}