package com.divillysausages.connected.state 
{
	import com.divillysausages.connected.constants.SoundConstants;
	import com.divillysausages.connected.logic.Pic;
	import com.divillysausages.connected.logic.Speech;
	import com.divillysausages.connected.logic.SpeechMsg;
	import com.divillysausages.connected.state.constants.StateNames;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import flash.events.MouseEvent;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class StateGameOver extends State 
	{
		
		/*******************************************************************************************/
		
		private var m_pic:Pic						= null;	// the pic of the guy talking to us
		private var m_speech:Speech					= null;	// the speech for our guy
		private var m_textDie:Vector.<SpeechMsg>	= null;	// the different strings that we're going to start off with
		private var m_textNotDie:Vector.<SpeechMsg>	= null;	// the different strings for when we miss a planet (all)
		private var m_textCurr:Vector.<SpeechMsg>	= null;	// our current speech
		private var m_speechCount:int				= 0;	// where we are in the list of speech messages
		private var m_speechWaitTime:Number			= 0.0;	// the wait time for the speech
		private var m_speechCurrTime:Number			= 0.0;	// the current wait time for the speech
		
		/*******************************************************************************************/
		
		public function StateGameOver( fsm:Game ) 
		{
			super( fsm, StateNames.STATE_GAME_OVER );
			
			// create our pic
			this.m_pic 			= new Pic;
			var totalW:Number	= this.m_pic.width + 310.0;
			this.m_pic.x		= DS.stage.stageWidth * 0.5 - totalW * 0.5;
			this.m_pic.y		= 100.0;
			
			// create our speech
			this.m_speech 	= new Speech( 300.0 );
			this.m_speech.x = this.m_pic.x + this.m_pic.width + 10.0;
			this.m_speech.y = this.m_pic.y;
			
			// create our speech text
			this.m_textDie = new Vector.<SpeechMsg>;
			this.m_textDie.push( new SpeechMsg( "Congratulations <SUBJECT_NAME>! You have crashed.", 3.0 ) );
			this.m_textDie.push( new SpeechMsg( "You have also killed billions, and are dead.", 5.0 ) );
			this.m_textDie.push( new SpeechMsg( "Because of this, you are terminated, job-wise. Life-wise, it is of course, also the case.", 5.0 ) );
			this.m_textDie.push( new SpeechMsg( "A cleanup crew has been dispatched to ensure plausible deniability, press SPACE to signal them", -1.0 ) );
			
			this.m_textNotDie = new Vector.<SpeechMsg>;
			this.m_textNotDie.push( new SpeechMsg( "Congratulations <SUBJECT_NAME>! You have been blown up for being very poor at your job.", 5.0 ) );
			this.m_textNotDie.push( new SpeechMsg( "Press SPACE to signal a cleanup crew to recover your body parts. Have a nice day", -1.0 ) );
		}
		
		override public function begin():void 
		{
			// add our speech and score
			DS.canvas.addChild( this.m_pic );
			DS.canvas.addChild( this.m_speech );
			
			// get our right vector
			this.m_textCurr = ( StatePlay.DID_DIE ) ? this.m_textDie : this.m_textNotDie;
			
			// reset our score etc
			this.m_pic.show( this.m_pic.x, this.m_pic.y );
			this.m_speechCount 		= 0;
			this.m_speechWaitTime	= 0.0;
			DS.callLater( this._showNextMessage, 300 );
			
			// add our click listener to show the text immediately
			DS.stage.addEventListener( MouseEvent.CLICK, this._showSpeechImmediately );
		}
		
		override public function end():void 
		{	
			// clear our speech
			this.m_speech.clear();
			
			// remove the other stuff
			this.m_pic.parent.removeChild( this.m_pic );
			this.m_speech.parent.removeChild( this.m_speech );
			
			// clear our listener
			DS.stage.addEventListener( MouseEvent.CLICK, this._showSpeechImmediately );
			Input.unbind( Key.SPACE );
		}
		
		override public function update(dt:Number):void 
		{				
			// update our speech text
			if ( this.m_speechCount <= this.m_textCurr.length && this.m_speechWaitTime > 0.0 && this.m_speech.isFullyShown )
			{
				this.m_speechCurrTime += dt;
				if ( this.m_speechCurrTime >= this.m_speechWaitTime )
					this._showNextMessage();
			}
		}
		
		/*******************************************************************************************/
		
		private function _spaceToNextMessage():void
		{
			DS.soundManager.playSound( SoundConstants.SFX_SELECT, 0, 0.5 );
			this._showNextMessage();
		}
		
		// shows the next message in line
		private function _showNextMessage():void
		{
			// unbind any keys
			Input.unbind( Key.SPACE );
			
			// check our count index
			if ( this.m_speechCount >= this.m_textCurr.length )
			{
				this.m_speech.hide();
				this.m_fsm.changeState( StateNames.STATE_PLAY );
				return;
			}
			
			// show the next message
			var msg:SpeechMsg = this.m_textCurr[this.m_speechCount++];
			this._showMsg( msg.msg, msg.waitTime );
			if ( this.m_speechWaitTime <= 0.0 )
				Input.bind( Key.SPACE, this._spaceToNextMessage );
		}
		
		// shows a message
		private function _showMsg( msg:String, waitTime:Number ):void
		{
			if ( this.m_speechWaitTime == -1.0 ) // wait time is -1, so do nothing
				return;
			this.m_speechCurrTime 	= 0.0;
			this.m_speechWaitTime	= waitTime;
			this.m_speech.show( msg );
		}
		
		// show the speech now
		private function _showSpeechImmediately( e:MouseEvent = null ):void
		{
			this.m_speech.showImmediately();
		}
		
	}

}