package com.divillysausages.connected.state
{
	import com.divillysausages.connected.assets.Resources;
	import com.divillysausages.connected.constants.SoundConstants;
	import com.divillysausages.connected.state.constants.StateNames;
	import com.divillysausages.ds.DS;
	
	/**
	 * The init state that creates everything
	 * @author Damian Connolly
	 */
	public class StateInit extends State 
	{
		
		/*******************************************************************************************/
		
		private var m_inited:Boolean = false;// have we inited the game
		
		/*******************************************************************************************/
		
		/**
		 * Creates the init state
		 * @param fsm The Game finite state machine
		 */
		public function StateInit( fsm:Game )
		{
			super( fsm, StateNames.STATE_INIT );
			
			// create the resources that we'll use in the game
			new Resources;
		}
		
		/**
		 * Inits our game
		 */
		override public function begin():void 
		{
			if ( this.m_inited )
			{
				DS.error( this, "We've already inited the game" );
				this.m_fsm.changeState( StateNames.STATE_PLAY );
				return;
			}
			
			var m:Main = ( DS.main as Main );
			
			// we've initied
			this.m_inited = true;
			
			// add all our sounds
			DS.assetManager.registerEmbeddedAsset( SoundConstants.SFX_TEXT, sText, false );
			DS.assetManager.registerEmbeddedAsset( SoundConstants.SFX_SUCCESS, sSuccess, false );
			DS.assetManager.registerEmbeddedAsset( SoundConstants.SFX_MISS, sMiss, false );
			DS.assetManager.registerEmbeddedAsset( SoundConstants.SFX_EXPLOSION, sExplosion, false );
			DS.assetManager.registerEmbeddedAsset( SoundConstants.SFX_SELECT, sSelect, false );
			
			// go to our check state
			CONFIG::debug 	{ this.m_fsm.changeState( StateNames.STATE_PLAY ); }
			CONFIG::release	{ this.m_fsm.changeState( StateNames.STATE_DS ); }
		}
		
	}

}